/*
  ------------------------------------------------------------------------------

    Copyright © 2005-2011, Tommi T. Järvi
                           tommi . jarvi @ iki . fi (remove spaces)


    This file is part of one of my programs.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,
    as published by the Free Software Foundation.

    This is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  ------------------------------------------------------------------------------
*/


#include "atomtypes.h"


void Atom_Types::setmass(int i, double m) {
  mass[i] = m;
}


Atom_Types::Atom_Types() {

  /*
    Default constructor for Atom_Types. Recompile to change
    maximum atom type. (Types are in the range 0...maxtypes-1.)
  */


  // --- variables
  int i;


  // --- set defaults
  maxtypes = 10;  // recompile to change
  ntypes = 0;


  // --- resize arrays
  atypename.resize(maxtypes);

#if defined(DEBUG)
  cout << "Atom_Types::Def. constr. (AT): P: natype mass types, maxtypes ntypes ";
  cout << natype << " " << mass << " " << types << " " << maxtypes << " " << ntypes << "...";
#endif

  natype = new int[maxtypes];
  mass = new double[maxtypes];
  Z = new int[maxtypes];
  types = new int[maxtypes];


  // --- make valgrind happy
  for(i=0;i<maxtypes;i++) {
    natype[i] = 0;
    mass[i] = 0;
    Z[i] = 0;
    types[i] = 0;
    atypename[i] = "";
  }

#if defined(DEBUG)
  cout << " ...done(AT)." << endl;
#endif
}


Atom_Types::~Atom_Types() {

  if(natype != NULL) delete [] natype;
  if(mass != NULL) delete [] mass;
  if(Z != NULL) delete [] Z;
  if(types != NULL) delete [] types;
  natype = NULL;
  mass = NULL;
  Z = NULL;
  types = NULL;
}


void Atom_Types::reset() {

  /*
    Set ntypes to 0 and reset everything
  */


  // --- variables
  int i;


  // --- reset
  for(i=0;i<maxtypes;i++) {
    types[i] = 0;
    atypename[i] = "";
    mass[i] = 0;
    Z[i] = 0;
    natype[i] = 0;
  }

  ntypes = 0;
}


void Atom_Types::zero() {

  /*
    Set number of atoms of each type to zero but keep
    atom types and masses etc.
  */


  // --- variables
  int i;


  // --- reset
  for(i=0;i<maxtypes;i++) {
    natype[i] = 0;
  }
}


void Atom_Types::setmasses(bool warn) {

  /*
    Set masses and atomic numbers of some known atom types.

    For warn = true, don't stop at error if not found, just set to zero.
  */


  // --- variables
  int i, j;
  int values;  // number of element data
  vector<string> masstype;  // atom types
  valarray<double> massmass;  // masses
  valarray<int> massZ;


  // --- data
  values = 117;
  masstype.resize(values);
  massmass.resize(values);
  massZ.resize(values);

masstype[  0 ] =      "Xx";
massmass[  0 ] =    0.0;
massZ[  0 ] =   0;

masstype[  1 ] =       "H";
massmass[  1 ] =    1.00794;
massZ[  1 ] =   1;
 
masstype[  2 ] =      "He";
massmass[  2 ] =    4.00260;
massZ[  2 ] =   2;
 
masstype[  3 ] =      "Li";
massmass[  3 ] =    6.94100;
massZ[  3 ] =   3;
 
masstype[  4 ] =      "Be";
massmass[  4 ] =    9.01219;
massZ[  4 ] =   4;
 
masstype[  5 ] =       "B";
massmass[  5 ] =   10.81100;
massZ[  5 ] =   5;
 
masstype[  6 ] =       "C";
massmass[  6 ] =   12.01070;
massZ[  6 ] =   6;
 
masstype[  7 ] =       "N";
massmass[  7 ] =   14.00674;
massZ[  7 ] =   7;
 
masstype[  8 ] =       "O";
massmass[  8 ] =   15.99940;
massZ[  8 ] =   8;
 
masstype[  9 ] =       "F";
massmass[  9 ] =   18.99840;
massZ[  9 ] =   9;
 
masstype[ 10 ] =      "Ne";
massmass[ 10 ] =   20.17970;
massZ[ 10 ] =  10;
 
masstype[ 11 ] =      "Na";
massmass[ 11 ] =   22.98977;
massZ[ 11 ] =  11;
 
masstype[ 12 ] =      "Mg";
massmass[ 12 ] =   24.30500;
massZ[ 12 ] =  12;
 
masstype[ 13 ] =      "Al";
massmass[ 13 ] =   26.98154;
massZ[ 13 ] =  13;
 
masstype[ 14 ] =      "Si";
massmass[ 14 ] =   28.08550;
massZ[ 14 ] =  14;
 
masstype[ 15 ] =       "P";
massmass[ 15 ] =   30.97376;
massZ[ 15 ] =  15;
 
masstype[ 16 ] =       "S";
massmass[ 16 ] =   32.06600;
massZ[ 16 ] =  16;
 
masstype[ 17 ] =      "Cl";
massmass[ 17 ] =   35.45270;
massZ[ 17 ] =  17;
 
masstype[ 18 ] =      "Ar";
massmass[ 18 ] =   39.94800;
massZ[ 18 ] =  18;
 
masstype[ 19 ] =       "K";
massmass[ 19 ] =   39.09830;
massZ[ 19 ] =  19;
 
masstype[ 20 ] =      "Ca";
massmass[ 20 ] =   40.07800;
massZ[ 20 ] =  20;
 
masstype[ 21 ] =      "Sc";
massmass[ 21 ] =   44.95591;
massZ[ 21 ] =  21;
 
masstype[ 22 ] =      "Ti";
massmass[ 22 ] =   47.86700;
massZ[ 22 ] =  22;
 
masstype[ 23 ] =       "V";
massmass[ 23 ] =   50.94150;
massZ[ 23 ] =  23;
 
masstype[ 24 ] =      "Cr";
massmass[ 24 ] =   51.99610;
massZ[ 24 ] =  24;
 
masstype[ 25 ] =      "Mn";
massmass[ 25 ] =   54.93805;
massZ[ 25 ] =  25;
 
masstype[ 26 ] =      "Fe";
massmass[ 26 ] =   55.84500;
massZ[ 26 ] =  26;
 
masstype[ 27 ] =      "Co";
massmass[ 27 ] =   58.93320;
massZ[ 27 ] =  27;
 
masstype[ 28 ] =      "Ni";
massmass[ 28 ] =   58.69340;
massZ[ 28 ] =  28;
 
masstype[ 29 ] =      "Cu";
massmass[ 29 ] =   63.54600;
massZ[ 29 ] =  29;
 
masstype[ 30 ] =      "Zn";
massmass[ 30 ] =   65.39000;
massZ[ 30 ] =  30;
 
masstype[ 31 ] =      "Ga";
massmass[ 31 ] =   69.72300;
massZ[ 31 ] =  31;
 
masstype[ 32 ] =      "Ge";
massmass[ 32 ] =   72.61000;
massZ[ 32 ] =  32;
 
masstype[ 33 ] =      "As";
massmass[ 33 ] =   74.92160;
massZ[ 33 ] =  33;
 
masstype[ 34 ] =      "Se";
massmass[ 34 ] =   78.96000;
massZ[ 34 ] =  34;
 
masstype[ 35 ] =      "Br";
massmass[ 35 ] =   79.90400;
massZ[ 35 ] =  35;
 
masstype[ 36 ] =      "Kr";
massmass[ 36 ] =   83.80000;
massZ[ 36 ] =  36;
 
masstype[ 37 ] =      "Rb";
massmass[ 37 ] =   85.46780;
massZ[ 37 ] =  37;
 
masstype[ 38 ] =      "Sr";
massmass[ 38 ] =   87.62000;
massZ[ 38 ] =  38;
 
masstype[ 39 ] =       "Y";
massmass[ 39 ] =   88.90585;
massZ[ 39 ] =  39;
 
masstype[ 40 ] =      "Zr";
massmass[ 40 ] =   91.22400;
massZ[ 40 ] =  40;
 
masstype[ 41 ] =      "Nb";
massmass[ 41 ] =   92.90638;
massZ[ 41 ] =  41;
 
masstype[ 42 ] =      "Mo";
massmass[ 42 ] =   95.94000;
massZ[ 42 ] =  42;
 
masstype[ 43 ] =      "Tc";
massmass[ 43 ] =   98.00000;
massZ[ 43 ] =  43;
 
masstype[ 44 ] =      "Ru";
massmass[ 44 ] =  101.07000;
massZ[ 44 ] =  44;
 
masstype[ 45 ] =      "Rh";
massmass[ 45 ] =  102.90550;
massZ[ 45 ] =  45;
 
masstype[ 46 ] =      "Pd";
massmass[ 46 ] =  106.42000;
massZ[ 46 ] =  46;
 
masstype[ 47 ] =      "Ag";
massmass[ 47 ] =  107.86820;
massZ[ 47 ] =  47;
 
masstype[ 48 ] =      "Cd";
massmass[ 48 ] =  112.41100;
massZ[ 48 ] =  48;
 
masstype[ 49 ] =      "In";
massmass[ 49 ] =  114.81800;
massZ[ 49 ] =  49;
 
masstype[ 50 ] =      "Sn";
massmass[ 50 ] =  118.71000;
massZ[ 50 ] =  50;
 
masstype[ 51 ] =      "Sb";
massmass[ 51 ] =  121.76000;
massZ[ 51 ] =  51;
 
masstype[ 52 ] =      "Te";
massmass[ 52 ] =  127.60000;
massZ[ 52 ] =  52;
 
masstype[ 53 ] =       "I";
massmass[ 53 ] =  126.90447;
massZ[ 53 ] =  53;
 
masstype[ 54 ] =      "Xe";
massmass[ 54 ] =  131.28999;
massZ[ 54 ] =  54;
 
masstype[ 55 ] =      "Cs";
massmass[ 55 ] =  132.90546;
massZ[ 55 ] =  55;
 
masstype[ 56 ] =      "Ba";
massmass[ 56 ] =  137.32700;
massZ[ 56 ] =  56;
 
masstype[ 57 ] =      "La";
massmass[ 57 ] =  138.90550;
massZ[ 57 ] =  57;
 
masstype[ 58 ] =      "Ce";
massmass[ 58 ] =  140.11600;
massZ[ 58 ] =  58;
 
masstype[ 59 ] =      "Pr";
massmass[ 59 ] =  140.90765;
massZ[ 59 ] =  59;
 
masstype[ 60 ] =      "Nd";
massmass[ 60 ] =  144.24001;
massZ[ 60 ] =  60;
 
masstype[ 61 ] =      "Pm";
massmass[ 61 ] =  145.00000;
massZ[ 61 ] =  61;
 
masstype[ 62 ] =      "Sm";
massmass[ 62 ] =  150.36000;
massZ[ 62 ] =  62;
 
masstype[ 63 ] =      "Eu";
massmass[ 63 ] =  151.96400;
massZ[ 63 ] =  63;
 
masstype[ 64 ] =      "Gd";
massmass[ 64 ] =  157.25000;
massZ[ 64 ] =  64;
 
masstype[ 65 ] =      "Tb";
massmass[ 65 ] =  158.92534;
massZ[ 65 ] =  65;
 
masstype[ 66 ] =      "Dy";
massmass[ 66 ] =  162.50000;
massZ[ 66 ] =  66;
 
masstype[ 67 ] =      "Ho";
massmass[ 67 ] =  164.93031;
massZ[ 67 ] =  67;
 
masstype[ 68 ] =      "Er";
massmass[ 68 ] =  167.25999;
massZ[ 68 ] =  68;
 
masstype[ 69 ] =      "Tm";
massmass[ 69 ] =  168.93420;
massZ[ 69 ] =  69;
 
masstype[ 70 ] =      "Yb";
massmass[ 70 ] =  173.03999;
massZ[ 70 ] =  70;
 
masstype[ 71 ] =      "Lu";
massmass[ 71 ] =  174.96700;
massZ[ 71 ] =  71;
 
masstype[ 72 ] =      "Hf";
massmass[ 72 ] =  178.49001;
massZ[ 72 ] =  72;
 
masstype[ 73 ] =      "Ta";
massmass[ 73 ] =  180.94791;
massZ[ 73 ] =  73;
 
masstype[ 74 ] =       "W";
massmass[ 74 ] =  183.84000;
massZ[ 74 ] =  74;
 
masstype[ 75 ] =      "Re";
massmass[ 75 ] =  186.20700;
massZ[ 75 ] =  75;
 
masstype[ 76 ] =      "Os";
massmass[ 76 ] =  190.23000;
massZ[ 76 ] =  76;
 
masstype[ 77 ] =      "Ir";
massmass[ 77 ] =  192.21700;
massZ[ 77 ] =  77;
 
masstype[ 78 ] =      "Pt";
massmass[ 78 ] =  195.07800;
massZ[ 78 ] =  78;
 
masstype[ 79 ] =      "Au";
massmass[ 79 ] =  196.96655;
massZ[ 79 ] =  79;
 
masstype[ 80 ] =      "Hg";
massmass[ 80 ] =  200.59000;
massZ[ 80 ] =  80;
 
masstype[ 81 ] =      "Tl";
massmass[ 81 ] =  204.38330;
massZ[ 81 ] =  81;
 
masstype[ 82 ] =      "Pb";
massmass[ 82 ] =  207.20000;
massZ[ 82 ] =  82;
 
masstype[ 83 ] =      "Bi";
massmass[ 83 ] =  208.98038;
massZ[ 83 ] =  83;
 
masstype[ 84 ] =      "Po";
massmass[ 84 ] =  209.00000;
massZ[ 84 ] =  84;
 
masstype[ 85 ] =      "At";
massmass[ 85 ] =  210.00000;
massZ[ 85 ] =  85;
 
masstype[ 86 ] =      "Rn";
massmass[ 86 ] =  222.00000;
massZ[ 86 ] =  86;
 
masstype[ 87 ] =      "Fr";
massmass[ 87 ] =  223.00000;
massZ[ 87 ] =  87;
 
masstype[ 88 ] =      "Ra";
massmass[ 88 ] =  226.00000;
massZ[ 88 ] =  88;
 
masstype[ 89 ] =      "Ac";
massmass[ 89 ] =  227.00000;
massZ[ 89 ] =  89;
 
masstype[ 90 ] =      "Th";
massmass[ 90 ] =  232.03810;
massZ[ 90 ] =  90;
 
masstype[ 91 ] =      "Pa";
massmass[ 91 ] =  231.03587;
massZ[ 91 ] =  91;
 
masstype[ 92 ] =       "U";
massmass[ 92 ] =  238.02890;
massZ[ 92 ] =  92;
 
masstype[ 93 ] =      "Np";
massmass[ 93 ] =  237.00000;
massZ[ 93 ] =  93;
 
masstype[ 94 ] =      "Pu";
massmass[ 94 ] =  244.00000;
massZ[ 94 ] =  94;
 
masstype[ 95 ] =      "Am";
massmass[ 95 ] =  243.00000;
massZ[ 95 ] =  95;
 
masstype[ 96 ] =      "Cm";
massmass[ 96 ] =  247.00000;
massZ[ 96 ] =  96;
 
masstype[ 97 ] =      "Bk";
massmass[ 97 ] =  247.00000;
massZ[ 97 ] =  97;
 
masstype[ 98 ] =      "Cf";
massmass[ 98 ] =  251.00000;
massZ[ 98 ] =  98;
 
masstype[ 99 ] =      "Es";
massmass[ 99 ] =  252.00000;
massZ[ 99 ] =  99;
 
masstype[100 ] =      "Fm";
massmass[100 ] =  257.00000;
massZ[100 ] = 100;
 
masstype[101 ] =      "Md";
massmass[101 ] =  258.00000;
massZ[101 ] = 101;
 
masstype[102 ] =      "No";
massmass[102 ] =  259.00000;
massZ[102 ] = 102;
 
masstype[103 ] =      "Lr";
massmass[103 ] =  262.00000;
massZ[103 ] = 103;
 
masstype[104 ] =      "Rf";
massmass[104 ] =  261.00000;
massZ[104 ] = 104;
 
masstype[105 ] =      "Db";
massmass[105 ] =  262.00000;
massZ[105 ] = 105;
 
masstype[106 ] =      "Sg";
massmass[106 ] =  263.00000;
massZ[106 ] = 106;
 
masstype[107 ] =      "Bh";
massmass[107 ] =  264.00000;
massZ[107 ] = 107;
 
masstype[108 ] =      "Hs";
massmass[108 ] =  265.00000;
massZ[108 ] = 108;
 
masstype[109 ] =      "Mt";
massmass[109 ] =  268.00000;
massZ[109 ] = 109;
 
masstype[110 ] =      "Ds";
massmass[110 ] =  271.00000;
massZ[110 ] = 110;
 
masstype[111 ] =      "Rg";
massmass[111 ] =  272.00000;
massZ[111 ] = 111;
 
masstype[112 ] =     "Uub";
massmass[112 ] =  285.00000;
massZ[112 ] = 112;
 
masstype[113 ] =     "Uut";
massmass[113 ] =  284.00000;
massZ[113 ] = 113;
 
masstype[114 ] =     "Uuq";
massmass[114 ] =  289.00000;
massZ[114 ] = 114;
 
masstype[115 ] =     "Uup";
massmass[115 ] =  288.00000;
massZ[115 ] = 115;
 
masstype[116 ] =     "Uuh";
massmass[116 ] =  292.00000;
massZ[116 ] = 116;
 
  // --- set masses

  for(i=0;i<ntypes;i++) {

    for(j=0;j<values;j++) {
      if(atypename[types[i]] == masstype[j]) {
	mass[types[i]] = massmass[j];
        Z[types[i]] = massZ[j];
	cout << "Set mass of " << atypename[types[i]] << " to " << mass[types[i]] << " and atomic number to " << Z[types[i]] << "." << endl;
	break;
      }
      if(j==values-1) {
	if(!warn) {
	  cerr << "Atom_Types::setmasses(): Unknown type " << atypename[types[i]] << endl;
	  exit(-1);
	}
	else {
	  cout << "  Warning: Unknown type " << atypename[types[i]] << ", setting mass to zero."<< endl;
	}
      }
    }
  }
}


Atom_Types::Atom_Types(const Atom_Types &syst) {

  int i;

#if defined(DEBUG)
  cout << "Atom_Types::copy constructor...";
#endif

  maxtypes = syst.maxtypes;  // recompile to change

  ntypes = syst.ntypes;
  atypename.resize(maxtypes);
  if(natype != NULL) delete [] natype;
  if(mass != NULL) delete [] mass;
  if(Z != NULL) delete [] Z;
  if(types != NULL) delete [] types;
  natype = new int[maxtypes];
  mass = new double[maxtypes];
  Z = new int[maxtypes];
  types = new int[maxtypes];

  for(i=0;i<maxtypes;i++) {
    atypename[i] = syst.atypename[i];
    natype[i] = syst.natype[i];
    mass[i] = syst.mass[i];
    Z[i] = syst.Z[i];
    types[i] = syst.types[i];
  }

#if defined(DEBUG)
  cout << "  ...done." << endl;
#endif
}


Atom_Types& Atom_Types::operator=(const Atom_Types &syst) {

  int i;

  if(this == &syst) return *this;

#if defined(DEBUG)
  cout << "Atom_Types::Operator=...";
#endif

  maxtypes = syst.maxtypes;  // recompile to change

  ntypes = syst.ntypes;
  atypename.resize(maxtypes);
  if(natype != NULL) delete [] natype;
  if(mass != NULL) delete [] mass;
  if(Z != NULL) delete [] Z;
  if(types != NULL) delete [] types;
  natype = new int[maxtypes];
  mass = new double[maxtypes];
  Z = new int[maxtypes];
  types = new int[maxtypes];

  for(i=0;i<maxtypes;i++) {
    atypename[i] = syst.atypename[i];
    natype[i] = syst.natype[i];
    mass[i] = syst.mass[i];
    Z[i] = syst.Z[i];
    types[i] = syst.types[i];
  }

#if defined(DEBUG)
  cout << "  ...done." << endl;
#endif

  return *this;
}


void Atom_Types::printtypes() {

  /*
    Print out atom types and number of atoms of each type.
  */


  // --- variables
  int i;  // loops


  // --- print
  cout << "    Atom types in system:" << endl;
  for(i=0;i<ntypes;i++) {
    cout << "      type " << types[i] << " (" << atypename[types[i]]
	 <<  "): " << natype[types[i]] << endl;
  }
}


void Atom_Types::add(string name, int &n) {

  /*
    Add an atom type of name "name" and number "n".
    if n==-1, set n automatically if type "name" doesn't exist

    If n=-1 and type exists, n gives the atom type number on output.

    sets: (doesn't count atoms to natype)

    ntypes, atypename, natype, types, mass (0)
    ------------------------------------------
    vector<string> atypename;  // names of atom types, 0, ..., highest_type
    (this should be reset to 0) int ntypes;  // number of atom types
    int *types;  // atom type numbers, 0, ..., ntypes-1 (not in any particular order)
    int *natype;  // number of atoms of a type, 0, ..., highest_type
  */

  int i;  // loops

  valarray<int> tempint;
  vector<string> tempstring;
  bool new_type;


  // --- check n

  if(n >= maxtypes) {
    cerr << "Atom_Types::add(): Trying to add atype " << n << " but highest poss. type " << maxtypes-1 << endl;
    exit(-1);
  }


  // --- check that atom type really is new

  new_type = true;

  // automatic type number
  if(n==-1) {
    for(i=0;i<ntypes;i++) {
      if(atypename[types[i]]==name) {
	new_type = false;
	n = types[i];
	break;
      }
    }
    if(new_type) {

      // set n automatically
      n = ntypes+1;
      if(n >= maxtypes) {
	cerr << "Atom_Types::atype_add(): Trying to add (autom.) atype " << n << " but highest poss. type " << maxtypes-1 << endl;
	exit(-1);
      }
    }
  }

  // type number specified
  else {
    for(i=0;i<ntypes;i++) {
      if(types[i]==n) {
	new_type = false;
	if(atypename[n]!=name) {
	  cerr << "Atom_Types::atype_add(): Trying to add atom of type " << n << " " << name << " but already have type " << types[i] << " " << atypename[types[i]] << endl;
	  exit(-1);
	}
	break;
      }
    }
  }


  // --- not new type

  if(!new_type) {
    return;
  }


  // --- new type

  types[ntypes] = n;
  ntypes++;
  atypename[n] = name;
  natype[n] = 0;
  mass[n] = 0;
  Z[n] = 0;

  // report
  cout << "  Added atom type " << name << " with number " << types[ntypes-1] << endl;
}


int Atom_Types::typenumber(string name) {

  /*
    Gives the atom type number of atom type "name". If there
    are more than one corresponding numbers, gives one of
    them.
  */


  // --- variables
  int i;
  int at = -1;


  // --- find number
  for(i=0;i<ntypes;i++) {
    if(atypename[types[i]]==name) {
      at = types[i];
      break;
    }
  }


  // --- check
  if(at==-1) {
    cerr << "Atom_Types::typenumber(): Error: No such atom type: " << name << endl;
    exit(-1);
  }


  return at;
}


string Atom_Types::name(int n) {

  /*
    Return the name of atom type n.
  */


  // --- check
  if(n > ntypes) {
    cerr << "Atom_Types::name(): Error: n ntypes " << n << " " << ntypes << endl;
    exit(-1);
  }


  // --- return
  return atypename.at(n);
}


int Atom_Types::freenumber() {

  /*
    Give the next free atom type number.
  */


  // --- variables
  int i, j;  // loops
  bool found;  // found?
  int result;  // free number found


  // --- find number
  found = false;
  result = -1;
  while(!found) {
    result++;
    found = true;
    for(i=0;i<ntypes;i++) {
      if(types[i]==result) {
	found = false;
	break;
      }
    }
  }  // found


  return result;
}


bool Atom_Types::safeadd(string name, int n) {

  /*
    Is it safe to add type (name, n)?

    1. (name, n) exists => true (is safe!)
    2. (name, n) name exists but n different => false (is not safe!)
    3. (name, n) n exists but name different => false
    4. (name, n) neither name nor n exists => true

    Could be relaxed on the 2. point.
  */


  // --- variables
  int i;  // loops


  // does n exist? (1, 3)
  for(i=0;i<ntypes;i++) {
    // n exists
    if(types[i]==n) {
      if(atypename[i]==name) {
	return(true);  // it's safe (1)
      }
      else {
	return(false);  // not safe (3)
      }
    }
  }

  /*
  // go on only if we still have a chance
  if(!result) {
    return result;
  }
  */

  // n does not exist
  for(i=0;i<ntypes;i++) {
    // name exists
    if(atypename[i]==name) {
      if(types[i]==n) {
	cerr << "Atom_Types::safeadd(): Error: This is a bug, report it." << endl;
	exit(-1);
      }
      return(false);  // not safe (2)
      break;
    }
  }


  // neither name or n exists (4)
  return true;
}
