.PU
.TH EULER l local

.SH NAME
euler \- Analysis and modification of atomic data

.SH SYNOPSIS
.B euler <input system> [options] [commands]

.SH DESCRIPTION
.P
Euler is a program for analysing and processing atomic data, for use
with atomistic simulations such as molecular dynamics or DFT. For
users familiar with the field, it provides an efficient command line
tool for pre- and post-processing of modelling results.
.P
The idea is that after giving the input system and possible options, the
commands are executed on the input system in the order they are given
on the command line, so that different analyses and operations can be
combined flexibly.

To get started, have a look at the "examples" section towards the
end. Reading the "notes" section is recommended as well.

.SH INPUT
.P
The input system can be specified in different ways. For input files
with more than one frame of atoms, the commands are executed for each
frame separately, as if they were separate files. The input methods
are:

.SS
xyz <file> <mode>
Read in an xyz-file.
.TP 7
mode
columns (example atom Cu)
.TP
0
Cu x y z
.TP
1
Cu x y z atype
  atype - integer atom type, negative for a fixed atom
.TP
2
Cu x y z atype atindex vx vy vz
  atindex - atom index, starting from 1
  v - velocity (in Å/fs)
.TP
3
Cu x y z V (see output mode 1)
.P
(Additional columns are ignored.)

.SS
lmpcdump <file>
Read in a LAMMPS "custom dump" file. Currently the positions,
velocities, atom type and atom and molecule indeces are read in, if
they are specified. Thus the atom indexes should stay unchanged, not
the order in which they appear in the file.

.SS
basis <file> <mode> <f> <nx ny nz>
Read in a basis file and create the system with nx*ny*nz unit
cells. Of the options, only the periodic boundary conditions can be
set after creating the system, not the box size which is calculated
automatically. Note also that the actual commands do not support other
than rectangular boundary conditions, although the basis can have any
unit cell vectors. The unit cell vectors are multiplied by the scaling
factor f. 
.P
The basis file is almost an ordinary xyz-file. The format is
.IP
nbasis
.br
basis vectors
.br
first atom
.br
and so on
.br
last atom
.P
For example for Mo (bcc) with lattice constant 3.15 Å:
.IP
2
.br
3.15 0.0 0.0 0.0 3.15 0.0 0.0 0.0 3.15
.br
Mo 0.0 0.0 0.0 1
.br
Mo 0.5 0.5 0.5 1
.P
The basis vectors can be specified in different modes:
.TP 5
1
All vector components as in the example above.
.TP
2
Specifying a, b, c, and the angles alpha, beta, and gamma, where the
vectors are then
.br
v1 = a*X
.br
v2 = b*( X cos(gamma) + Y sin(gamma) )
.br
v3 = c*( X cos(beta)  + Y cos(alpha) + Z sin(beta) sin(alpha) )
.TP
3
Same as 1 except the coordinates are cartesian and not with respect to
the basis vectors.

.SH OPTIONS
.P
The options have to be specified before the commands.

.SS
pbc <x y z>
Periodic boundary conditions:
.IP 0
no
.IP 1
yes

.SS
box <x y z>
Box size.
.P
Note: Non-periodic systems are the default but it is safest to specify
the periodics explicitly. It's good to always specify a box size, as
the cells used in constructing the neighbour list use it. In the
absense of a box size, many operations will be terribly slow.

.SH COMMANDS - BASIC INFORMATION

.SS
temperature
Print out temperature and other stuff.
.P
The stuff printed out is temperature (T), the radius of gyration
(R_g), and the kinetic energy of center of mass.

.SS
limits
Print the minimum and maximum coordinate values in each direction.

.SS
cmekin
Print information on center of mass velocity and kinetic energy.

.SH COMMANDS - CONTROL

.SS
df <i>
Only process following commands for every i'th frame (but always for
the first and last).

./" These don't work at the moment
./".SS
./"begint <t>
./"Discard frames with time smaller than t. (This assumes we've been able
./"to read PARCAS-style xyz files or something else with the time in the
./"comment line.)

./".SS
./"endt <t>
./"Discard frames with time smaller than t. See also notes for begint.

./".SS
./"dt <t>
./"Only take frames with an interval larger than dt. See also notes for
./"begint.
./"dummy to get even number of quotations

.SH COMMANDS - BASIC TRANSFORMATIONS

.SS
center
Move the geometric center of the system to the origin.

.SS
centerv
Remove center of mass velocity from the system.

.SS
rotate <phi theta psi>
Euler rotation. The ranges for the angles are
.IP phi
[0,2pi]
.IP theta
[0,pi]
.IP psi
[0,2pi]
.P
If theta is chosen such that theta = acos(u), where
u is a uniform random number in the range [-1,1], then
at least a unit vector will point to a uniformly random point
on the unit sphere after the rotation. The third degree of
freedom has not been tested.
.P
Hints:
.TP 2
-
To rotate [111] along [001], use (phi,theta,psi) = (-3pi/4,
atan(-sqrt(2)), 0).

.SS
rotateaxis <phi theta delta>
Rotate by angle delta around the axis (phi, theta).

.SS
translate <x y z>
Translate all atoms, including fixed ones.

.SS
movepos
Move system so that all atoms have non-negative coordinates.

.SS
scale <f>
Scale coordinates by f. Don't touch box size etc.

.SS
addek <E>
Add E eV/at of kinetic energy to the system by scaling
velocities. (Velocities assumed in Å/fs.)

.SS
addveldz <vel z>
Add velocity (vel in A/fs) to atoms above z in the z-direction.

.SS
moveatom <n x y z>
Move atom n. With N atoms, the atom numbers are [0,N-1].

.SS
removeat <n mode>
Remove atom n. The atoms are indexed in the range [0, N-1], where N is
the total number of atoms. The mode is 1 or 2: 1 replaces the removed
atom by the last atom while 2 keeps the order of the remaining atoms
unchanged. 1 is of course faster.

.SS
removetype <n>
Remove atoms of type n. Be careful if you use an input xyz mode that
doesn't fix the atom type number as otherwise the numbers are assigned
automatically. (And can thus differ for frames in a single movie-file,
if the atomic content does not stay constant, and in the same order.)

.SS
removename <name>
Remove atoms by name. This should be safer than removetype. Note
that at the moment, names are case sensitive. This will hopefully be
changed at some point.

.SS
removerand <n>
Remove n atoms randomly.

.SS
removeV <a b>
Remove atoms with potential energy (V) in ]a,b[.

.SS
moveV <a b x y z>
Move atoms with potential energy (V) in ]a,b[ by [x y z]. This is
useful in place of removing when visualizing with programs where each
frame has to have the same atoms.

.SS
removeK <a b>
Remove atoms with kinetic energy (K) in ]a, b[.

.SS
remove <dir 1/0 cut>
Remove atoms above (1) or below (0) cut in dir {0,1,2}.

.SS
removehs <bx by bz dx dy dz>
Remove atoms with (r-b) \\cdot d > 0.

.SS
removedist <cut x y z>
Remove atoms farther away than cut from (x,y,z).

.SS
removecoord <cut min max>
Remove atoms with coordination number (within cut) between min and max.

.SS
removedat <cut n min>
Remove atoms farther away than cut from atom n (don't go below min
atoms).

.SS
join <file>
Join a system from file to this system. The atom type numbers must not
overlap, but the program should give a reasonable error message if
they do.

.SS
ontop <dist file>
Join a system from file to this system so that the new system is on
top (in z) with dist in between.

.SH COMMANDS - CLUSTERING

.SS
cluster <cut>
Cluster atoms with cutoff. You need to do this before some other
commands, such as removing the largest cluster.

.SS
leavelargest
Remove atoms not in the largest block. This assumes the clustering has
already been done.

.SS
removelargest
Remove atoms in the largest block. This assumes the clustering has
already been done.

.SS
removeblocksizes <n1 n2>
Remove atoms in blocks of size from n1 to n2.

.SS
cldist
Output cluster size distributions and statistics. Good for, e.g.,
analysing sputtering yield after removing the substrate using
removelargest. See examples.

.SH COMMANDS - ANALYSIS

Most of the analysis routines output some information on the screen
and place atomwise results or identifications in the potential energy
-array (V-array) which can be output using a proper xyz output mode.

.SS
spheret <cut>
This will calculate the temperature and then replace the kinetic
energy (=temperature) of each atom with a value averaged over the
respective atom and it's neighbours within the cutoff.

.SS
getmoved <rcut file>
Compares to another xyz-file and removes atoms that have moved by
less than rcut.

.SS
sro <a at nei zmin zmax>
Get SRO-parameter, e.g. sro 3.8 Cu Ag -100 100. The short range order
parameter eta = n_1/n_2, where
.IP n1
average number of neighbours of type nei of atoms of type at
.IP n2
average total number of neighbours of atoms of type at
.IP a
lattice constant, the cutoff for the neighbours includes the nearest
neighbours: cut=a*(1/sqrt(2) + 1)/2. So this assumes fcc but of course
you can scale a to get the right cutoff for any system.

.SS
ssro <a at nei>
Get smart SRO-parameter, e.g. ssro 3.8 Au Au for Cu3Au. The smart
SRO-parameter is SRO = 1 - eta/c_A, where c_A is the concentration of
atoms of type nei and eta is defined above.
.IP "SRO -> 1"
ordered phase, Au atoms far from each other
.IP "SRO ~= 0"
random alloy
.IP "SRO -> -3"
segregation, Au atoms gather together

.SS
lsro <a at nei>
Calculates the SRO-parameter for each atom separately and stores in
the V-array.

.SS
sfactor <a nx ny nz>
Get the stucture factor |S|^2 with Miller indices n, with S defined as

  S = (1/N) \\sum_{i=0}^{N-1} exp(i k \\cdot x_i),

where N is the number of atoms and k = (2*pi/a)*n, a being the lattice
constant. Hint: To get the n=(1,1,1) structure factor for an fcc
system with [1,1,1] along z, take n=(0,0,1) and divide a by sqrt(3).

.SS
sfactorz <a zlim nx ny nz>
Get stucture factor for the atoms above z = zlim normalised to those
below.

.SS
pair <a b dr>
Get pair correlation function with r=[a,b] with a step of dr. (Not
tested much.)

.SS
art
Output amplitude for R(t), where R is the distance of an atom from the
center of mass of the system. This is one of the few commands which
requires a movie-file.

.SS
cna <cut mode>
Common neighbour analysis as described in Honeycutt and Andersen,
J. Phys. Chem. 91, 4950 (1987). The parameters are:
.IP cut
Bond cutoff.
.IP "mode=1"
Only pairs with bonds between the atoms.
.IP "mode=2"
Same as 1 but mark all surface atoms (identified by coordination
number) with 5. Note that if a known signature with low coordination
is specified in the input file, the atom is identified according to
that signature, not as a surface atom.
.P
Atoms are identified based on their cna-signatures. Each bond is given
a signature and atoms are identified if their signatures match known
ones. The identifying number for atom i is placed
in the potential energy array so use a proper xyz-mode for output
(mode 1). (A legend is output to standard output.)
.P
Statistics of the raw CNA bond signatures is always output.
.P
The known bond signatures are read in from the file
cna_signatures.input, the format of which is
.P
---------
.br
2                   number of signatures
.br
1 2                 id for 1st sig. + number of bonds
.br
comment line
.br
1 2 3 4             1st bond of 1st sign. (nn n b t)
.br
3 2 4 5
.br
2 3                 id for 2nd sig. + number of bonds
.br
and so forth
.br
----------------
.br
.P
Above, the bond numbers are
.IP nn
1 if atoms are neighbours, 0 otherwise
.IP n
number of common neighbours
.IP b
bonds between common neighbours
.IP t
topology
.P
When given a negative cutoff, the command also prints out a file
(cna_signatures.output) containing both the known signatures and
unknown signatures detected in the analysis. This file can then be
used as input.

.SS
hbond <cut a b t1 t2 t3>
Hydrogen bond analysis.
.IP cut
Cutoff for bonds. Should be large enough so that the distance from all
of the three participating atoms to one another is inside the cutoff.
.IP "a b"
Minimum and maximum angle for hydrogen bond, in the range [0,180]
degrees.
.IP "t1 t2 t3"
Search for bonds between atoms of these types.
.P
For example for systems where the atom types (numbers) are C (1), H
(2) and O (3) and one wants hydrogen bonds of type OH--O, with OHO
angles above 120 degrees and O-O distance below 3.0 Å, one would use
.br
hbond 3.0 120 180 3 2 3
.P
Note that a warning is printed to standard error output if more than
one hydrogen bond is found between an atom pair, but the analysis is
not stopped.

.SS
qsolid <cut thres frac l>
Identify solid / liquid atoms based on q-correlation, as described in
Mendez-Villuendas and Bowles, Phys. Rev. Lett. 98, 185503 (2007). The
parameters are:
.IP cut
neighbour cutoff
.IP thres
threshold for identifying two atoms as connected ([0,1]?, the paper
uses 0.65-0.68 but 0.15 has worked for me in case of metals. 0.3 seems
to be good for diamond.)
.IP frac
fraction of neighbours required to be connected for an atom to be
identified as solid (0.5 seems to work most of the time)
.IP l
the spherical harmonic l (the paper uses l=6)
.P
The identification (1=solid, 0=liquid) is again placed in the
potential energy array that can be output with xyz-mode 1.
Note that in the paper, the correlation number c_ij is apparently
assumed real. I'm haven't checked that and thus use |c_ij| instead to
identify connected pairs.

.SS qcorr <cut l>
Gives the q-correlation numbers (from which the connectedness of atoms
is determined in the qsolid-command above) for all bonds.

.SS
coordination <cut>
Determine the number of neighbours inside cut for each atom and place
in the potential energy array V[i]. Also output statistics on the
coordination numbers found.

.SS
bdef <...>
Bond defect analysis. For each atom, bonds to neighbouring atoms are
analyzed, and if the result is not what is specified on the command
line, the value 1 is placed in the potential energy array V[i] (see
output xyzmode 1). Likewise, for matching bonds the value is 0. The
required bonds are specified at the command line like so:
.P
First the number of atom types for which bonds are analyzed. (All
unspecified atoms will get the value 0). For each atom type the number
of neighbours and the type of the atom are then specified, and then
for each neighbour, three values: type, bond cutoff, and number of
required bonds. Complicated, huh?
.P
For example, for polyethylene, we require that each C have 2 H and 2 C
neighbours and each H only 1 C neighbour. That's accomplished by
.br
bdef 2 [ 2 C (H 1.3 2) (C 1.9 2) ] [ 2 H (H 1.3 0) (C 1.3 1) ]
.br
where the brackets are just to help reading, not part of the syntax.

.SS
mdef <...>
Molecule analysis. This is similar to "bdef", except that whole
molecules are analyzed. Prior to this command, the system should be
separated into molecules either by reading in a format where they are
specified or by performing cluster analysis with the "cluster"
command.
.P
The command tries to match each molecule to the given pattern and
places the value 0 in the V-array of atoms in that molecule if
succesfull. For molecules not matching the pattern, the value is 1.
.P
The molecule pattern consists of the different atoms in a molecule:
The molecule should have N atoms of type A, which all have certain
neighbours. Note that this algorithm will fail to distinguish some
cases. For example the position of a side-chain in a polymer does not
alter the pattern.
.P
The input is best illustrated with an example. This line will identify
ethylene carbonate molecules:
.nf
  mdef 5   1 O 1 C 1.7   1 C 1.7 3 O 1.7 O 1.7 O 1.7   2 O 2 C 1.7 C 1.7   2 C 4 O 1.7 C 1.7 H 1.7 H 1.7   4 H 1 C 1.7
       |                 | |  |  | |  |  |  |  |  |
       |                 | |  |  | |  |  >  >  >  >                                 etc. etc.
       |                 | |  |  | >  >                 of which one O within 1.7 A
       |                 | |  >  > with 3 neighbours (disregarding anything farther than 1.7 A)
       |                 | >   C
       |                 > one
       > Five types of atoms
.fi

.SS
findvac <a dr f>
Find vacancies in the system by inserting spheres.
.IP a
Lattice constant
.IP dr
Vacancy trial grid spacing (approximate)
.IP f
Spheres will have this fraction of the nearest neighbour distance as
their radius.
.P
The vacancies are found by inserting spheres of radius r = f*r_nn,
where r_nn = a/sqrt(2). (So it's easy to specify for fcc, otherwise
there's nothing specific to fcc.) The algorithm is thus:
.IP 1.
For non-periodic systems, center system for maximum
efficiency. (Because of this and the next point, the original system
is not unchanged.)
.IP 2.
For non-periodic directions, determine box size based on farthest
atoms from the origin.
.IP 3.
Go through the box using a grid with the approximate spacing
dr. Whenever a sphere fits into a grid point, create a vacancy
there. (Previous vacancies prevent creation of overlapping new ones.) 
.IP 4.
Cluster the vacancies found and remove the largest one, corresponding
(usually at least) to the vacuum part of the system.
.IP 5.
Save two files: vacancies.xyz and vacancies_and_system.xyz.
.P
For platinum, findvac 3.92 0.1 0.8 has usually worked fine for
me. Also note that the algorithm is not very fast, although it has
been optimized from the original, very slow, version.

.SH COMMANDS - MISCELLANEOUS

.SS
distz <x y>
Get distance of closest atom from line (x,y) || z.

.SS
distfrom <x y z>
Get distance of each atom from point (x,y,z).

.SS
runaveV <dir a b dx delta>
Get running average of the V-array for V in [a:dx:b] with bins of
width delta. (Not tested much.)

.SH COMMANDS - OUTPUT

.SS
savexyz <file mode>
Save xyz-file. See below for the modes.
.TP 7
mode
columns (example atom Cu)
.TP
0
plain xyz
.TP
1
plain xyz + potential energy
.TP
2
Cu x y z atype atindex vx vy vz
  atype - atom type, negative for a fixed atom
  atindex - atom index, starting from 1
  v - velocity (in Å/fs)
.TP
3
Cu x y z atype atindex
.TP
4
Cu x y z atype V block
.TP
5
Cu x y z block
.TP
6
Cu x y z T
  T of fixed atoms will be wrong!
.TP
7
Cu x y z atype atindex V
.P
(Note that atom types will be negative if any degree of freedom is fixed.)



.SS
savexyzfr <file mode>
Save xyz-file of each frame with name fileN.xyz, N in [1,#frames].

.SS
savexyzfrc <mode>
Save xyz-file of each frame. The files are named according to the
comment line of the system, as [comment].xyz. For example for input
xyz-files, this is the second line.

.SS
savepov <file m minT maxT>
Save pov-file.
.IP "m=0"
positions only
.IP "m=1"
colours from temperature
.IP "m=2"
colours from potential energy
.IP minT
min temperature or potential energy (the value shown in blue)
.IP maxT
max temperature or potential energy (the value shown in red)
.P
Note that this file format is a simple POV-Ray file for input into a
POV-Ray input script.

.SS
savepovfr <file m maxT>
Save pov-file frames with names fileN.pov. For options, see savepov.

.SS
saveposcar[s] <file a order>
Save poscar-file (for VASP)
.IP a
lattice constant, or scaling factor
.IP order
order in which to output atom types: #types t1 t2 etc., e.g.: 2 0 1
.P
With saveposcars, save poscar-file with "Selective dynamics" input.

.SS
savebgf <file mode>
Save bgf-file compatible with ReaxFF.
.IP mode
1 - BIOGRF (non-periodic), 2 - XTLGRF (periodic)

.SS
savebgfd <file mode desc>
Save bgf-file compatible with ReaxFF.
.IP mode
see above
.IP desc
DESCRP description

.SS
savelmp <file mode>
Save LAMMPS data file. The modes are:
.TP
1              atomic
.TP
2              atomic + velocities
.TP
3              charge
.TP
4              charge + velocities
.TP
5              molecular
.TP
6              molecular + velocities



.SH NOTES

.P
Before using, please read this and the "todo" section below.
.IP Disclaimer:
This program has been made mainly for my own use. That means I have
implemented many commands that I've only used once so there might be
bugs in them. Also, although I've tried to make the behaviour of the
commands intuitive (to me), you might find surprising effects. So be
careful. Bug reports and suggestions are welcome, of course.
.IP "Large systems:"
The memory consumption for common neighbour analysis (tested on
30.11.2007) was around 0.18 kB/atom.
.IP "Atom indices:"
A proper handling of atom indeces, independent of the indeces of the
programmed arrays, should be implemented. Now some commands use a
range of [1,N] and others [0,N-1]. It is also not possible to have
indeces higher than the number of atoms in the system. In any case,
it's always best to check that the output is as expected.

.SH EXAMPLES

.P
Let's say you've simulated xenon bombardment of a platinum particle on
a carbon substrate and want to get the sputtering yield. Depending on
what you exactly want, you would do something like this:
.P
euler xyz input.xyz 0 pbc 1 1 0 box 50.0 50.0 100.0 removename C
removename Xe cluster 5.0 removelargest cldist
.P

.SH TODO

.IP 1.
Atom types should be made case-insensitive.
.IP 2.
Currently, only the number of command parameters is checked, not
whether they make sense.
.IP 3.
Reading in xyz-files, if the element in the first column is preceded
by a space, it is read in wrong by the default C++ input. This should
be corrected.

.SH AUTHORS

Tommi Järvi, firstname.surname@iki.fi (but replace ä->a).
.br
See also www.iki.fi/tommi.jarvi/
